
#ifndef __KERNEL_H__
#define __KERNEL_H__


typedef unsigned char UCHAR;
typedef unsigned short USHORT;
typedef unsigned int UINT;
typedef unsigned long ULONG;
typedef unsigned short WORD;
typedef unsigned char BYTE;
typedef unsigned long DWORD;
typedef int BOOL;

#define TRUE 1
#define FALSE 0

UCHAR __inbyte(USHORT Port);
void __outbyte(USHORT Port, UCHAR Data);

#pragma intrinsic(__inbyte)
#pragma intrinsic(__outbyte)

#define dd(x)                          \
	__asm _emit  (x)       & 0xff  \
	__asm _emit  (x) >> 8  & 0xff  \
	__asm _emit  (x) >> 16 & 0xff  \
	__asm _emit  (x) >> 24 & 0xff

#define KERNEL_STACK   0x00103fff
#define KERNEL_START   0x00101000
#define KERNEL_LENGTH  0x001039a0 //0x0000200F

#define MAGIC          0x1BADB002
#define FLAGS          1 << 16
#define CHECKSUM       -(MAGIC + (FLAGS))
#define HEADER_ADR     0x00101000
#define LOAD_ADR       0x00101000
#define LOAD_END_ADR   0x001039a0 //0x0010200F
#define BSS_END_ADR    0x001039a0 //0x0010200F
#define ENTRY_ADR      0x00101030
#define MODE_TYPE      0x00000000
#define WIDTH          0x00000000
#define HEIGHT         0x00000000
#define DEPTH          0x00000000

void main(ULONG, ULONG);

struct regs
{
	UINT ds, es, fs, gs;
	UINT edi, esi, ebp, esp, ebx, edx, ecx, eax;
	UINT int_no, err_code;
	UINT eip, cs, eflags, useresp, ss;    
};

void irq_install();
void irq_install_handler(int irq, void (*handler)(struct regs *r));

void cls();
void putch(UCHAR c);
void puts(UCHAR *text);

void *memset(void *dest, char val, int count);
void *memcpy(void *dst_ptr, const void *src_ptr, unsigned count);
void idt_set_gate(UCHAR num, ULONG base, USHORT sel, UCHAR flags);
USHORT *memsetw(USHORT *dest, USHORT val, int count);
int strlen(const char *str);
void keyboard_install();
void isrs_install();
void gdt_install();
void idt_install();
void timer_install();
void init_video(void);


#endif  // __KERNEL_H__
